/* Copyright (C) 2024 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer] */

/// @file   KfFramework.cxx
/// @brief  The Kalman-filter framework main class (header)
/// @since  28.03.2024
/// @author Sergei Zharko <s.zharko@gsi.de>

#include "KfFramework.h"

using cbm::algo::kf::Framework;

// ---------------------------------------------------------------------------------------------------------------------
//
template<typename DataT>
bool Framework<DataT>::Init()
try {
  std::stringstream errMsg;
  try {
    fPars.Init();
  }
  catch (const std::runtime_error& err) {
    errMsg << "\tParameters initialization errors: " << err.what() << '\n';
  }

  try {
    fSetup.Init();
  }
  catch (const std::runtime_error& err) {
    errMsg << "\tSetup initialization errors: " << err.what() << '\n';
  }

  if (!errMsg.str().empty()) {
    throw std::runtime_error(errMsg.str());
  }

  L_(info) << '\n' << fPars.ToString(3) << '\n' << fSetup.ToString(3);
  return true;
}
catch (const std::exception& err) {
  L_(error) << "kf::Framework: initialization failed. Reason: " << err.what();
  return false;
}

template class cbm::algo::kf::Framework<float>;
template class cbm::algo::kf::Framework<double>;
// template class cbm::algo::kf::Framework<fvec>;
